/*
 * NET Thermometer UDP server
 *
 *  Copyright (C) 1999-2001  Claudio Lanconelli
 *                                                                     
 *  e-mail: lancos@libero.it                                          
 *  http://www.LancOS.com                                         
 *
 * $Id: udpserver.cpp,v 1.3 2001/04/30 20:30:03 lanconel Exp $
 */

//-------------------------------------------------------------------------//
//                                                                         //
// This program is free software; you can redistribute it and/or           //
// modify it under the terms of the GNU  General Public License            //
// as published by the Free Software Foundation; either version2 of        //
// the License, or (at your option) any later version.                     //
//                                                                         //
// This program is distributed in the hope that it will be useful,         //
// but WITHOUT ANY WARRANTY; without even the implied warranty of          //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       //
// General Public License for more details.                                //
//                                                                         //
// You should have received a copy of the GNU  General Public License      //
// along with this program (see COPYING);     if not, write to the         //
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. //
//                                                                         //
//-------------------------------------------------------------------------//

#include <string.h>
#include <errno.h>

#include "../readtemp.h"
#include "inet.h"

struct param {
	char	help;
	int		port;
	int		com;
};

/*
 * Parse the command line
 */
int getargs(int argc, char *argv[], struct param *parameters)
{
	int n = 1;
	int errore = 0;

	while(n < argc)
	{
		if ( strcmp(argv[n], "-help") == 0 ||
			 strcmp(argv[n], "-h") == 0 ||
			 strcmp(argv[n], "-?") == 0 )
			parameters->help = 1;
		else
		if (strcmp(argv[n], "-port") == 0)
		{
			if (++n < argc)
				if((parameters->port = atoi(argv[n])) < 1024)
					errore = 1;
		}
		else
		if (strcmp(argv[n], "-com") == 0)
		{
			if (++n < argc)
				parameters->com = atoi(argv[n]);
			if (parameters->com < 1 || parameters->com > 4)
				errore = 1;
		}
		n++;
	}
	return errore;
}

void SocketEnd();
int SocketInit();
char *GetSockErrorString(int val);

int main(int argc, char *argv[])
{
	SOCKET				sockfd;
	struct sockaddr_in	serv_addr, cli_addr;
	static struct param	parameters = {0, SERV_PORT, SERV_COM};

	/*
	 * Check for parameters
	 */
	if(getargs(argc, argv, &parameters) != 0)
		myerror(1,"server: port number must be > than 1023");

	printf("NetThermometer (Server) by LancOS 1999-2001 " VERSION "\n\n");

	if (parameters.help != 0)
	{
		printf(	"Usage: %s [-help] [-port port] [-com port]\n", argv[0]);
		printf(	"       -help show this help\n"
				"       -port service port number (%d)\n", SERV_PORT);
		printf(	"       -com serial com port number (%d)\n\n", SERV_COM);
	}
	else
	{
		/*
		 * Initialize Sockets
		 */
		if ( !SocketInit() )
		{
			fprintf(stderr, "error in Socket initialization\n");
			exit(1);
		}

		/*
		 * Open the UDP socket
		 */
		sockfd = socket(AF_INET, SOCK_DGRAM, 0);
		if (sockfd == INVALID_SOCKET)
		{
#ifdef	WIN32
			errno = WSAGetLastError();
			fprintf(stderr, "%s\n", GetSockErrorString(errno));
#endif
			myerror(1, "server: can't open datagram socket");
		}

		/*
		 * Bind to the local address to receive requests
		 */
		bzero((char *)&serv_addr, sizeof(serv_addr));
		serv_addr.sin_family      = AF_INET;
		serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
		serv_addr.sin_port        = htons(parameters.port);

		if (bind(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0)
			myerror(1, "server: can't bind local address");

		printf(	"Socket Port: %d\n"
				"Com Port: COM%d\n\n",
			parameters.port,
			parameters.com);

		while (1)
		{
			ThermoRequest request;
			ThermoReply reply;
			socklen_t clilen;
			int err = 0;

			/*
			 * Wait for the request from clients
			 */
			clilen = sizeof(cli_addr);
			if (recvfrom(sockfd, (char *)&request, sizeof(request), 0, (struct sockaddr *)&cli_addr, &clilen) != sizeof(request))
				myerror(1, "server: recvfrom error");

			bzero(&reply, sizeof(reply));

			/*
			 * Check the request
			 */
			if (strcmp(request.Id, THERMOID) != 0)
			{
				reply.result = -1;
				strcpy(reply.msg, "Error: bad request\n");
			}
			else
			{
				if (request.first < 1 || request.first > N_THERM)
					err = 1;
				if (request.last < 1 || request.last > N_THERM)
					err = 1;
				if (request.first > request.last)
					err = 1;

				if (!err)
				{
					if (request.Action == ACTION_GET)
					{
					//	printf("Received valid request (GET)\n");

						/*
						 * Get the temperatures
						 */
						ReadTemperature(reply.status, reply.vals, request.first, request.last, parameters.com);

						int k;
						for (k = 0; k < N_THERM; k++)
							reply.vals[k] = htons(reply.vals[k]);
						strcpy(reply.msg, "Ok");
					}
					else
					if (request.Action == ACTION_MEAN)
					{
					//	printf("Received valid request (MEAN)\n");

						/*
						 * Calculate the Mean of temperatures
						 */
						ReadTemperature(reply.status, reply.vals, request.first, request.last, parameters.com);

						/*
						 * Calculate the mean
						 */
						int ok = 1;
						int k;
						for (k = request.first-1; k < request.last; k++)
							if (reply.status[k] != 0)
								ok = 0;
						if (ok)
						{
							long sum = 0;
							int j = 0;
							for (k = request.first-1; k < request.last; k++)
							{
								sum += reply.vals[k];
								j++;
							}
							if (j)
							{
								reply.status[0] = 0;
								reply.vals[0] = (short)(sum / j);
							}
							else
							{
								reply.status[0] = -1;
								reply.vals[0] = 0;
							}
						}
						else
							reply.status[0] = -2;
						strcpy(reply.msg, "Ok");

						for (k = 0; k < N_THERM; k++)
							reply.vals[k] = htons(reply.vals[k]);
					}
					else
					{
						err = 1;
					}
				}

				if (err)
				{
					strcpy(reply.msg, "Error: bad parameters\n");
					reply.result = -2;
				}
			}

			/*
			 * Send the temperatures back to the client
			 */
			if (sendto(sockfd, (char *)&reply, sizeof(reply), 0, (struct sockaddr *)&cli_addr, clilen) != sizeof(reply))
				myerror(1, "server: sendto error");
		}
	}	
	return 0;
}
